/**
  * File: EventsQueueUser.java
  * Content: Interface to be implemented by EventsQueue Users
  * Author: LuisM Pena
  * Date: 16th May 2001
  * Version: 0.51.00
  * Last change:
  *
  *
  **/

package sensei.domains;

import sensei.middleware.gms.Message;
import sensei.middleware.gms.View;

/**
 * Interface to be implemented by EventsQueue Users
 * Note: the interface is public just for testing purposes
 */
public interface EventsQueueUser
{
  /**
   * The user of the EventsQueue can set the alarm in different ways, mainly throught the value returned
   * on the operation newViewEvent. If the alarm is activated and there are no events to send, this method
   * is called.
   * @returns true if the system could process the alarm. If false is returned
   */
  public void processAlarmEvent();

  /**
   * Event thrown after receiving an expuslsion event
   */
  public void processExpulsionEvent();

  /**
   * Event thrown to process the given PTP message
   */
  public void processPTPMessageEvent(int sender, Message message);

  /**
   * Event thrown to process the given cast message
   */
  public void processCastMessageEvent(int sender, Message message);

  /**
   * Event thrown after a changing view event
   * @return CONTINUE_BLOCKING_PREVIOUS to keep blocked the previous messages, if they would be blocked
   * @return DISCARD_PREVIOUS_MESSAGES to discard any previous messages. Alarm is activated
   * @return UNBLOCK_PREVIOUS_VIEW to process messages from the previous view. Alarm is activated
   * @return UNBLOCK_ALL_PREVIOUS_VIEWS to process messages from the previous views. Alarm is activated
   */
  public int changingViewEvent();

  /**
   * Event thrown after a view event. The view doesn't yet have to be installed, althought the state transfer
   * system can already see the changes and return therefore the right action to do with any blocked message.
   * @return CONTINUE_BLOCKING_PREVIOUS to keep blocked the previous messages
   * @return DISCARD_PREVIOUS_MESSAGES to discard any previous messages. Alarm is activated
   * @return UNBLOCK_PREVIOUS_VIEW to process messages from the previous view. Alarm is activated
   * @return UNBLOCK_ALL_PREVIOUS_VIEWS to process messages from the previous views. Alarm is activated
   */
  public int newViewEvent(View view);

  /**
   * Event thrown to install the new view
   */
  public void processViewEvent(View view);

  public static final int CONTINUE_BLOCKING_PREVIOUS = 1;
  public static final int DISCARD_PREVIOUS_MESSAGES = 2;
  public static final int UNBLOCK_ALL_PREVIOUS_VIEWS = 3;
  public static final int UNBLOCK_PREVIOUS_VIEW = 4;
}
