/**
  * File: STGroupMember.java
  * Content: StateTransferGroupMember, encapsulates the different GroupMember state transfer models to
  *          unify the access to the state transfer methods
  * Author: LuisM Pena
  * Date: 31st Octonber 2001
  * Version: 0.51.00
  * Last change:
  *
  **/

package sensei.domains;

import sensei.middleware.gms.GroupHandler;
import sensei.middleware.gms.GroupMember;
import sensei.middleware.gms.Message;
import sensei.middleware.gms.View;
import sensei.middleware.domains.BasicStateHandler;
import sensei.middleware.domains.Checkpointable;
import sensei.middleware.domains.DomainsNarrower;
import sensei.middleware.domains.PhaseCoordination;
import sensei.middleware.domains.PhaseCoordinationHolder;
import sensei.middleware.domains.State;
import sensei.middleware.domains.StateHandler;
import sensei.middleware.domains.StateTransferType;


/**
  * StateTransferGroupMember, encapsulates the different GroupMember state transfer models to
  * unify the access to the state transfer methods
  **/
abstract class STGroupMember
{
  public abstract GroupMember getGroupMember();
  public abstract void freeResources();
  public abstract boolean needsTransfer();
  public abstract StateTransferType getStateTransferType();

  //*************************************************************************************//
  //**************************** STATE TRANSFER METHODS *********************************//
  //*************************************************************************************//

  public abstract void processPTPMessage(int sender, Message msg) throws Exception;
  public abstract void processCastMessage(int sender, Message msg) throws Exception;
  public abstract void memberAccepted(int identity, GroupHandler handler, View theView) throws Exception;
  public abstract void changingView() throws Exception;
  public abstract void installView(View theView) throws Exception;
  public abstract void excludedFromGroup() throws Exception;

  //*************************************************************************************//
  //**************************** STATE TRANSFER METHODS *********************************//
  //*************************************************************************************//

  public abstract void assumeState() throws Exception;
  public abstract void startTransfer(int[] joiningMembers, PhaseCoordinationHolder phase) throws Exception;
  public abstract State getState(PhaseCoordinationHolder phase) throws Exception;
  public abstract void setState(State s, PhaseCoordination phase) throws Exception;
  public abstract void stopTransfer(int[] joiningMembers, boolean transferFinished) throws Exception;
  public abstract void syncTransfer(int coordinator, PhaseCoordinationHolder phase) throws Exception;
  public abstract void interruptTransfer(PhaseCoordinationHolder phase) throws Exception;
  public abstract void continueTransfer(int[] joiningMembers, PhaseCoordinationHolder coordinator,
                                        PhaseCoordination joining) throws Exception;

  //*************************************************************************************//
  //**************************** FACTORY ************************************************//
  //*************************************************************************************//

  public static STGroupMember createSTGroupMember(GroupMember member)
  {
    Trace.code("STGroupMember.java -> STGroupMember createSTGroupMember ( GroupMember member )");
    STGroupMember ret = null;
    StateHandler stateHandler = DomainsNarrower.toStateHandler(member);
    if (stateHandler==null)
    {
      BasicStateHandler basicStateHandler = DomainsNarrower.toBasicStateHandler(member);
      if (basicStateHandler==null)
      {
        ret = new STGroupMemberStateless(member);
        Checkpointable checkpointable = DomainsNarrower.toCheckpointable(member);
        if (checkpointable==null)
          ret = new STGroupMemberStateless(member);
        else
          ret = new STGroupMemberCheckpointable(checkpointable);
      }
      else
        ret = new STGroupMemberBasicStateHandler(basicStateHandler);
    }
    else
      ret=new STGroupMemberStateHandler(stateHandler);
    return ret;
  }
}

