/**
  * File: MapStringSetObserversHandler.java
  * Content: Helper class to handle Observers on the MapStringSet class
  *
  * Author: LuisM Pena
  * Date: 20th November 2001
  * Version: 0.29.00
  * Last change:
  *
  *
  **/

package sensei.gmns;

import sensei.util.Error;

import sensei.middleware.gmns.SetMemberInfo;
import sensei.middleware.gmns.MapStringSetObservable;
import sensei.middleware.gmns.MapStringSetObserver;
import sensei.middleware.util.ObjectsHandling;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

class MapStringSetObserversHandler
{
  public MapStringSetObserversHandler(MapStringSetObservable observable)
  {
    Trace.code("MapStringSetObserversHandler.java -> MapStringSetObserversHandler ( MapStringSetObservable observable )");
    observers = new ArrayList();
    this.observable = observable;
  }

  public synchronized void addObserver(MapStringSetObserver observer)
  {
    Trace.code("MapStringSetObserversHandler.java -> synchronized void addObserver ( MapStringSetObserver observer )");
    observers.add(observer);
  }

  public synchronized boolean removeObserver(MapStringSetObserver observer)
  {
    Trace.code("MapStringSetObserversHandler.java -> synchronized boolean removeObserver ( MapStringSetObserver observer )");
    Iterator it = observers.iterator();
    while (it.hasNext())
      if (ObjectsHandling.areEquivalent(observer, (MapStringSetObserver) it.next()))
      {
        it.remove();
        return true;
      }
    return false;
  }

  public synchronized void informPut(String str, SetMemberInfo info)
  {
    Trace.code("MapStringSetObserversHandler.java -> synchronized void informPut ( String str , SetMemberInfo info )");
    Iterator it = observers.iterator();
    while (it.hasNext())
      try
      {
        ((MapStringSetObserver) it.next()).putDone(observable, str, info);
      }
      catch(Exception ex)
      {
        Error.unhandledException(Consts.AREA,ex);
      }
  }

  public synchronized void informRemove(String str, SetMemberInfo info)
  {
    Trace.code("MapStringSetObserversHandler.java -> synchronized void informRemove ( String str , SetMemberInfo info )");
    Iterator it = observers.iterator();
    while (it.hasNext())
      try
      {
        ((MapStringSetObserver) it.next()).removeDone(observable, str, info);
      }
      catch(Exception ex)
      {
        Error.unhandledException(Consts.AREA,ex);
      }
  }

  public synchronized void informClear()
  {
    Trace.code("MapStringSetObserversHandler.java -> synchronized void informClear ( )");
    Iterator it = observers.iterator();
    while (it.hasNext())
      try
      {
        ((MapStringSetObserver) it.next()).clearDone(observable);
      }
      catch(Exception ex)
      {
        Error.unhandledException(Consts.AREA,ex);
      }
  }

  List observers;
  MapStringSetObservable observable;
};
