/**
  * File: SendViewChannelRequester.java
  * Content: Defines a channel requester to send a view
  * Author: LuisM Pena
  * Date: 1st August 2000
  * Version: 0.97.00
  * Last change:
  *
  **/

package sensei.gms;

import sensei.middleware.gms.InternalView;
import sensei.middleware.gms.InternalViewId;
import sensei.middleware.gms.SenseiGMSMember;
import sensei.middleware.gms.Token;


//*************************************************************************************//
//**************************** SEND VIEW CHANNEL REQUESTER ****************************//
//*************************************************************************************//

/**
  * Channel Requester used to send views
  **/
class SendViewChannelRequester implements ChannelRequester
{

  public SendViewChannelRequester(int thisMember)
  {
    Trace.code("SendViewChannelRequester.java -> SendViewChannelRequester ( int thisMember )");
    this.thisMember = thisMember;
  }

  public void setInfoToSend(InternalView view, Token token)
  {
    Trace.code("SendViewChannelRequester.java -> void setInfoToSend ( InternalView view , Token token )");
    this.view = view;
    this.token = token;
  }

  public boolean invokeMember(CommunicationChannel channel)
  {
    Trace.code("SendViewChannelRequester.java -> boolean invokeMember ( CommunicationChannel channel )");
    assert view!=null && token!=null && channel!=null;
    boolean ret;

    SenseiGMSMember channelMember = channel.getLink();

    assert channelMember!=null;

    try
    {
      ret=channelMember.receiveView(thisMember, view, token) && channel.isReliable();
    }
    catch(Exception ex)
    {
      ret=false;
      Trace.handledException(ex);
    }
    return ret;
  }

  int thisMember;
  Token token;
  InternalView view;
}
