/**
  * File: SenseiConf.java
  * Content: Class to show graphically the sensei configuration values
  * Author: LuisM Pena
  * Date: 16th December 2001
  * Version: 0.97.00
  * Last change:
  *
  **/

package sensei.util;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridLayout;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;

import java.io.IOException;
import java.util.HashMap;

public class SenseiConf extends JFrame
{

  public SenseiConf()
  {
    JTabbedPane tabbed = new JTabbedPane();
    getContentPane().add(tabbed);
    tabbed.add(createGMSPanel(),"GMS");
    tabbed.add(createGMNSPanel(),"GMNS");
    tabbed.add(createLoggingPanel(),"Logging");
    String title = Configuration.getSingleton().getConfigurationFile();
    setTitle(title==null? "No configuration file" :  title);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    pack();
    show();
  }

  public static void main(String args[])
  {
    if (args.length==0)
    {
      System.out.println("Reading default configuration");
      Configuration.getSingleton();
      new SenseiConf();
    }
    else if (args.length==1)
    {
      try
      {
        System.out.println("Reading configuration at " + args[0]);
        Configuration.getSingleton(args[0], new HashMap());
        new SenseiConf();
      }
      catch(IOException ex)
      {
        System.out.println(ex);
      }
    }
    else
      System.out.println("Only one parameter could be given to specify a non default configuration file");
  }

  static JPanel fixePanel(JPanel panel)
  {
    JPanel fixW = new JPanel(new BorderLayout());
    JPanel fixH = new JPanel(new BorderLayout());
    fixH.add(panel,BorderLayout.WEST);
    fixW.add(fixH,BorderLayout.NORTH);
    return fixW;
  }

  static void createPair(String label, JComponent jvalue, String tip, Container labels, Container values)
  {
    JLabel jlabel = new JLabel(label);
    jlabel.setToolTipText(tip);
    jvalue.setToolTipText(tip);
    if (jvalue instanceof JTextField)
      ((JTextField)jvalue).setEditable(false);
    labels.add(jlabel);
    values.add(jvalue);
  }

  static void createPair(String label, String value, String tip, Container labels, Container values)
  {
    createPair(label, new JTextField(value), tip, labels, values);
  }

  static void createPair(String label, int value, String tip, Container labels, Container values)
  {
    createPair(label, String.valueOf(value).toString(), tip, labels, values);
  }

  static void createPair(String label, boolean value, String tip, Container labels, Container values)
  {
    createPair(label, value? Boolean.TRUE.toString() : Boolean.FALSE.toString(), tip, labels, values);
  }

  public static JPanel createGMNSPanel()
  {
    JPanel ret = new JPanel(new BorderLayout());
    JPanel labels = new JPanel(new GridLayout(0,1,5,5));
    JPanel values = new JPanel(new GridLayout(0,1,5,5));
    ret.add(labels, BorderLayout.WEST);
    ret.add(values, BorderLayout.EAST);
    Configuration conf =  Configuration.getSingleton();

    createPair(Published, conf.isGMNSReferencePublishedBySocket()? PublishedByPort + conf.getGMNSReferencePort() :
      PublishedByFile + conf.getGMNSReferenceFile(), TipPublished, labels, values);
    createPair(BackupFile, conf.getGMNSBackupFile().toString(), TipBackupFile, labels, values);
    createPair(BackupPeriod, conf.getGMNSBackupPeriod(), TipBackupPeriod, labels, values);
    createPair(CheckPeriod, conf.getGMNSCheckPeriod(), TipCheckPeriod, labels, values);
    createPair(CollisionTimeout, conf.getGMNSCollisionTimeout(), TipCollisionTimeout, labels, values);

    Configuration.GMNShost[] hosts = conf.getGMNSHosts();
    JComponent associated;
    if (hosts.length==0)
      associated = new JTextField("No other hosts");
    else
      associated = new JComboBox(hosts);
    createPair(Hosts, associated, TipHosts, labels, values);

    return fixePanel(ret);
  }

  public static JPanel createGMSPanel()
  {
    JPanel ret = new JPanel(new BorderLayout());
    JPanel labels = new JPanel(new GridLayout(0,1,5,5));
    JPanel values = new JPanel(new GridLayout(0,1,5,5));
    ret.add(labels, BorderLayout.WEST);
    ret.add(values, BorderLayout.EAST);
    Configuration conf =  Configuration.getSingleton();

    createPair(ChannelLiveness, conf.getGMSChannelLiveness(), TipChannelLiveness, labels, values);
    createPair(MaxProcessMessageDelay, conf.getGMSMaxProcessMessageDelay(), TipMaxProcessMessageDelay, labels, values);
    createPair(TokenStoppedPeriod, conf.getGMSTokenLostTimeout(), TipTokenStoppedPeriod, labels, values);
    createPair(TokenLostTimeout, conf.getGMSTokenLostTimeout(), TipTokenLostTimeout, labels, values);
    createPair(WeakConsensus, conf.hasGMSWeakConsensus(), TipWeakConsensus, labels, values);

    return fixePanel(ret);
  }

  public static JPanel createLoggingPanel()
  {
    JPanel ret = new JPanel(new BorderLayout());
    JPanel labels = new JPanel(new GridLayout(0,1,5,5));
    JPanel values = new JPanel(new GridLayout(0,1,5,5));
    ret.add(labels, BorderLayout.WEST);
    ret.add(values, BorderLayout.EAST);
    Configuration conf =  Configuration.getSingleton();

    createPair(Logging, conf.hasLogging(), TipLogging, labels, values);
    createPair(RemoteLoggingPort, conf.getRemoteLoggingPort(), TipRemoteLoggingPort, labels, values);

    return fixePanel(ret);
  }

  //LOGGING
  final static String Logging = "Logging: ";
  final static String RemoteLoggingPort = "Remote logging port: ";
  //GMNS
  final static String Published = "Published: ";
  final static String BackupFile = "Backup file: ";
  final static String BackupPeriod = "Backup period: ";
  final static String CheckPeriod = "Check period: ";
  final static String CollisionTimeout = "Collision timeout: ";
  final static String Hosts = "GMNS instances: ";
  //GMS
  final static String ChannelLiveness = "Channel liveness: ";
  final static String MaxProcessMessageDelay = "Max delay processing messages: ";
  final static String TokenStoppedPeriod = "Token retencion period: ";
  final static String TokenLostTimeout = "Token lost timeout: ";
  final static String WeakConsensus = "Weak consensus: ";
  //LOGGING tips
  final static String TipLogging = "Specifies if the system handles logs entries or not";
  final static String TipRemoteLoggingPort = "Port where the logging server is located, or 0 to use local logging";
  //GMNS tips
  final static String TipPublished = "The GMNS is known by other GMNS instances publishing its reference on a URL or a port";
  final static String TipBackupFile = "File to store the information, only read if every GMNS instance falls down";
  final static String TipBackupPeriod = "Time in milliseconds to store the backup file";
  final static String TipCheckPeriod = "Time in milliseconds to check that the subscribed members in the GMNS are alive";
  final static String TipCollisionTimeout = "Time in milliseconds to wait in case that 2 GMNS instances collide during initialization";
  final static String TipHosts = "URLs or sockets where this GMNS will look for other GMNSs istances to form a group. Only used at startup time.";
  //GMS tips
  final static String TipChannelLiveness = "The period in milliseconds for a channel to give an answer before being considered faulty";
  final static String TipMaxProcessMessageDelay = "Period in milliseconds to process any GMS event before considering that the application is too slow and is excluded from its group";
  final static String TipTokenStoppedPeriod = "Time in milliseconds that the token is stopped on a member when there is no group activity";
  final static String TipTokenLostTimeout = "Period in milliseconds without any group communication to consider lost the token and to start its recovery";
  final static String TipWeakConsensus = "Using weak consensus, a group can survive after loosing a half of its members; it needs LANs resistent to partitions";

  final static String PublishedByFile = "by file, on ";
  final static String PublishedByPort = "on port number ";};
