/**
  * File: LoggingSession.java
  * @author LuisM Pena
  * Version: 0.97.00
  **/

package sensei.util.logging;

import java.io.DataInputStream;
import java.io.IOException;
import java.net.Socket;

/**
  * Class implementing each session with the LoggerServer.
  * Protocol:
  *   Inicio: INIT + PASSWORD
  *   Change name: CHANGE NAME + new Name
  *   Print: PRINT + line
  **/
public class LoggingSession extends Thread
{

  /*******************************************************************************/
  /**************************** CONSTRUCTOR **************************************/
  /*******************************************************************************/

  /**
    * A LoggingSession operates on a received socket.
    **/
  public LoggingSession(LoggerServer parent, Socket socket, long initialTime)
  {
    this.parent=parent;
    this.socket=socket;
    this.initialTime=initialTime;
    input=null;
    validated=false;
    logStream=null;

    message("Starting new logging session");

    try
    {
      input=new DataInputStream(socket.getInputStream());
      if (validateSession())
        start();
      else
        releaseResources();
    }
    catch(IOException ex)
    {
      releaseResources();
    }
  }

  /*******************************************************************************/
  /**************************** RELEASE RESOURCES ********************************/
  /*******************************************************************************/

  void releaseResources()
  {
    logStream.close();
    if (input!=null) try{input.close();}catch(IOException ex){}
    try{socket.close();}catch(IOException ex){}

    logStream=null;
    input=null;
    socket=null;
    parent=null;
  }

  /*******************************************************************************/
  /**************************** MESSAGE ******************************************/
  /*******************************************************************************/

  void message(String msg)
  {
    if (logStream!=null)
      parent.message("["+logStream + "] : " + msg);
    else
      parent.message("[@uncompleted session] : " + msg);
  }

  /*******************************************************************************/
  /**************************** VALIDATE SESSION *********************************/
  /*******************************************************************************/

  boolean validateSession()
  {
    try
    {
      if ((input.readInt()==INIT) && (input.readUTF().equals(PASSWORD)))
      {
        logStream = new FileLogStream(false);
        validated=true;
        message("Session validated");
      }
    }
    catch(IOException ex)
    {
      message ("Error validating session due to exception " + ex.getMessage());
      validated=false;
    }
    return validated;
  }

  /*******************************************************************************/
  /**************************** RUN **********************************************/
  /*******************************************************************************/

  /**
    * Thread method
    **/
  public void run()
  {
    try
    {
      while(true)
      {
        int op=input.readInt();
        String parameter=input.readUTF();
        if (op==CHANGE_NAME)
        {
          logStream.setLogName(parameter);
          message("Changed logging file name");
        }
        else if (op==PRINT)
        {
          logStream.println(String.valueOf(System.currentTimeMillis() - initialTime) + parameter);
        }
        else
        {
          message("Received invalid logging, closing the logging session");
          break;
        }
      }
    }
    catch(IOException ex)
    {
      message ("Closing session, received socket exception" + ex.getMessage());
      validated=false;
    }
    releaseResources();
  }

  /*******************************************************************************/
  /*******************************************************************************/
  /*******************************************************************************/

  Socket socket;
  DataInputStream input;
  boolean validated;
  FileLogStream logStream;
  LoggerServer parent;
  long initialTime;

  public final static int INIT=3483;
  public final static int CHANGE_NAME=17584;
  public final static int PRINT=38247;
  public final static String PASSWORD="LoggingSession";
}
